/**************************************************************************************************************
 *
 * I2C-Funktionsbibliothek
 *
 * Datei : I2C.h
 *
 *************************************************************************************************************/

#include	<pic18.h>
#include "defines.h"

#define SCL                   RC3                 // Pin SCL am EEPROM
#define SDA                   RC4                 // Pin SDA am EEPROM

#define SDA_PORT              16                  // Port C.4
#define SCL_PORT              8                   // Port C.3

#define	XTAL_FREQ	4MHZ		                        // Vlliger Quark. Mu erstmal so bleiben !
#define	MHZ	*1000			                            // number of kHz in a MHz

#define	DelayUs(x)	{ unsigned int _dcnt; \
			  _dcnt = (x)/(12MHZ/(XTAL_FREQ))|1; \
			  while(--_dcnt != 0) \
				  continue; }

#define READ                  FALSE
#define WRITE                 TRUE


//----------
// Adressen
//----------
#define Adress_RTC 					0xA2

//----------------------
// Funktionsprototypen
//----------------------

//-------------
// Allgemeines
//-------------
void i2c_init (void);                                                   // Macht die Schnittstelle startklar
char i2c_open (char Adresse, char Richtung);                            // Adressiert einen Slave, liefert TRUE bei ACK
void i2c_close (void);                                                  // Erzeugt die Stop-Condition
char i2c_writebyte (char byte);                                         // Schreibt zum Slave, TRUE bei ACK vom Slave
char i2c_readbyte (char ACK);                                           // Liest ein Byte

//----------------
// Testfunktionen
//----------------
char i2c_BusScan (char Adresse);                                      // Liefert True, wenn der Slave vorhanden ist

//----------------
// Uhrenfunktionen
//----------------
char i2c_Read_RTC (char dev, char Adresse);                           // Liest ein Byte aus der RTC aus
char i2c_Write_RTC (char dev, char Adresse, char Wert);               // Schreibt ein Byte in die RTC

//---------------
// IO-Funktionen
//---------------
char I2C_Output (char Adresse, char Wert);                            // Setzt den Output-Port
char I2C_Input (char Adresse);                                        // Liest den Input-Port

int i2c_ReadTemp (char Adresse);																			// Liest einen Temperatursensor aus

