/**************************************************************************************************************
 *
 * Bordmonitor
 *
 * Autor : Finger
 *
 * Controller : Microchip 18F6620, 10 Mhz
 *
 * TAB-Space : 2
 * Editor : MI-Edit
 * Entwicklungsumgebung : Hi-Tech PIC-C 18 (www.htsoft.com)
 *
 *************************************************************************************************************/

#include	<pic18.h>
#include	<stdlib.h>
#include	<stdio.h>
#include  <string.h>
#include  "defines.h"
#include  "serial.h"
#include  "i2c.h"
#include  "eep.h"

#define	DelayUsISR(x)	{ unsigned int _dcnt; \
			  _dcnt = (x)/(12MHZ/(XTAL_FREQ))|1; \
			  while(--_dcnt != 0) \
				  continue; }

//----------------------------------------------------------
// Rckgabepuffer fr INT -> ASCII-Wandlung und Textausgabe
//----------------------------------------------------------
char outpbuf [Buflen];
volatile char Wandelpuffer [20];
volatile char UARTString [80];
volatile char Zeichenpuffer;
volatile char Pufferzeiger = 0;
volatile char Zerlegung = FALSE;

// Zhlerstnde
unsigned int Handstarts = 0;
unsigned int Telestarts = 0;

//-------------------
// Drehgeberposition
//-------------------
volatile int DrehPosition = 0;
volatile int AltDrehPosition = 0;

//-------------
// Zeitmessung
//-------------
volatile unsigned long SystemTicker = 0;
volatile char SekundenToggle = FALSE;
volatile int Sekundenzaehler = 0;

//-----------
// Messwerte
//-----------
struct
 {
  float Spannung;
  float Strom;
  int BatterieTemp;
  int InnenraumTemp;
  int WasserTemp;
 } Daten;

#define MessBufLen 10

float Rohspannung [MessBufLen];
float Rohstrom [MessBufLen];
int UMessBufZeiger = 0;
int IMessBufZeiger = 0;

//-----------------
// Zustandsautomat
//-----------------
int Zustand = ZustandSchlafen;
char Rueckgabe = 0;

//-------------------
// zyklische Ausgabe
//-------------------
volatile char Temperaturanzeige = 0;
volatile char TemperaturanzeigeFix = FALSE;
volatile char Zyklenticker = 0;

//----------
// Timeouts
//----------
volatile int HeizZeit = 0;
volatile int LatenzZeit = 0;
volatile int BatMonTimeout = BatMonTimeoutDefault;

//------
// Test
//------
volatile unsigned int IntCount = 0;

//---------------------------------
// Konfiguration des Controllers
// Fr Kartendatei auskommentieren
//---------------------------------
__CONFIG(1, OSCSEN & HS);       // Quarz 10 Mhz mit PLL = 40 Mhz
__CONFIG(2, BORV45 & WDTEN);       // Wachkter, 4.5V BrownOut
__CONFIG(4, LVPDIS & STVREN);      // Kein LowVoltageProgramm, Reset bei StackOverflow

/*********************************************************************************
 *
 * Zerlegt den Textstring vom Batteriemonitor und zuppelt die Zahlenwerte da raus
 *
 *********************************************************************************/
void BatteriemonitorStringAusbeinen (void)
 {
  float zwert;
  char i;
  float swert;

  // Steuerzeichen entfernen
  i= 0;
  while ((UARTString [i]) && (i < 40))
   {
    if (UARTString [i] < 32) UARTString [i] = ' ';
    i++;
   }

  print (UART1, UARTString);
  print (UART1, "##\n\r");

if (UARTString [5] == 'U')
 {
  memset (Wandelpuffer, sizeof(Wandelpuffer), 0);
  strncpy (Wandelpuffer, UARTString+7, 2);
  Wandelpuffer [2] = '\0';
  zwert = atoi (Wandelpuffer);

  //print (UART1, Wandelpuffer);
  //print (UART1, "\n\r");


  // Spannung Nachkomma
  memset (Wandelpuffer, sizeof(Wandelpuffer), 0);
  strncpy (Wandelpuffer, UARTString+10, 1);
  Wandelpuffer [1] = '\0';
  zwert += ((float)atoi (Wandelpuffer)) / 10;

  // Plausibilittsprfung
  if ( (zwert > 18) || (zwert < 8) ) zwert = Fehlerwert;

  // In Puffer bernehmen und Mittelwert der letzten "MessBufLen" Werte bilden
  if (zwert != Fehlerwert)
   {
    Rohspannung [UMessBufZeiger] = zwert;
    UMessBufZeiger++;
    if (UMessBufZeiger >= MessBufLen) UMessBufZeiger = 0;
    zwert = 0;
    for (i = 0; i < MessBufLen; i++)
     {
      zwert += Rohspannung [i];
      // sprintf (outpbuf, " %d", (int)(Rohspannung [i]*10));
      // print (UART1, outpbuf);
     }
    Daten.Spannung = zwert / MessBufLen;
    // print (UART1, "\n\r");
   }

  //-------
  // Strom
  //-------

  // Strom Vorkomma
  memset (Wandelpuffer, sizeof(Wandelpuffer), 0);
  strncpy (Wandelpuffer, UARTString+16, 3);
  Wandelpuffer [3] = '\0';
  swert = atoi (Wandelpuffer);

  // Strom Nachkomma
  memset (Wandelpuffer, sizeof(Wandelpuffer), 0);
  strncpy (Wandelpuffer, UARTString+20, 1);
  Wandelpuffer [1] = '\0';

  zwert = atoi (Wandelpuffer);
  zwert /= 10;


  if (swert >=0) swert += zwert; else
  							 swert -= zwert;

  // Plausibilittsprfung
  if ((swert > 99) || (swert < -99)) swert = Fehlerwert;

  // In Puffer bernehmenund Mittelwert der letzten "MessBufLen" Werte bilden
  if (swert != Fehlerwert)
   {
    Rohstrom [IMessBufZeiger] = swert;
    IMessBufZeiger++;
    if (IMessBufZeiger >= MessBufLen) IMessBufZeiger = 0;
    zwert = 0;
    for (i = 0; i < MessBufLen; i++) zwert += Rohstrom [i];
    Daten.Strom = zwert / MessBufLen;
   }

  //--------------------
  // Batterietemperatur
  //--------------------
  memset (Wandelpuffer, sizeof(Wandelpuffer), 0);
  strncpy (Wandelpuffer, UARTString+26, 5);

  // print (UART1, Wandelpuffer);
  // print (UART1, " ");

  Daten.BatterieTemp = atoi (Wandelpuffer);

  // sprintf (outpbuf, " %d", Daten.BatterieTemp);
  // print (UART1, outpbuf);

  // Plausibilittsprfung
  if ((Daten.BatterieTemp > 150) || (Daten.BatterieTemp < -99)) Daten.BatterieTemp = Fehlerwert;

  BatMonTimeout = BatMonTimeoutDefault;

 }

  // Puffer wieder leeren
  memset (UARTString, sizeof (UARTString), 0);

  Zerlegung = FALSE;

 }

/***************
 *
 * Zentrale ISR
 *
 ***************/
interrupt void INT_(void)
 {

   //------------------
   // USART-Behandlung
   //------------------
   isr_usart ();

   //--------------------------------------------
   // Ist was vom Batteriemonitor reingekommen ?
   //--------------------------------------------
   if (RC2IF)
    {

      // Zeichen aufsammeln
        if ( (OERR) /*|| (FERR))*/) CREN = 0; else
        {
        Zeichenpuffer = RCREG2;
        if (Zeichenpuffer == '\0') Zeichenpuffer = ' ';
        UARTString [Pufferzeiger] = Zeichenpuffer;
        if (Pufferzeiger<sizeof (UARTString)) Pufferzeiger++;
        // if (Pufferzeiger<sizeof (UARTString)) Pufferzeiger++; else Pufferzeiger=0;
        }
        // Ende entdeckt ?
        if (Zeichenpuffer == 'C')
         {
          UARTString [Pufferzeiger] = '\0';
          Pufferzeiger = 0;
          Zerlegung = TRUE;			// Marker setzen das wieder was da ist
         } // Von if (Zeichenpuffer == '\r')

    } // From if (RCIF)

   //---------------------------
   // Druck-Drehgeber auswerten
   //---------------------------
   if (INT2IF)
    {
      IntCount++;

      // Prfen auf gltige Flanke
      DelayUsISR(50);																							// Kurze Pause
      if (!(Links))                                          // Immer noch Low ?
       { 																														// Immer noch Low
        if (Rechts) DrehPosition++; else DrehPosition--;      // Richtung bestimmen
       } else
       {
        if (Rechts) DrehPosition--; else DrehPosition++;      // Richtung bestimmen
       }

     INT2IF = 0;																									  // Flag rcksetzen
    }

   //--------------------
   // Timer 1 berlauf ?
   //--------------------
   if (TMR1IF)
   	{

      //-------------------
      // Ablauf alle 10 ms
      //-------------------
      TMR1IF=0;                                         // Clear Interrupt-Flag
      TMR1H=0xF3;                                       // n = 2^16 - t [s] * 625000
      TMR1L=0xCB;                                       // Timer 1 neu vorladen (10 ms)

      SystemTicker++;

      //---------------------
      // Ablauf jede Sekunde
      //---------------------
      Sekundenzaehler++;
      if (Sekundenzaehler >= 100)
       {

        if ( (BatMonTimeout > 0) && (BatMonEin) ) BatMonTimeout--;


        Sekundenzaehler = 0;
        SekundenToggle = TRUE;
        Zyklenticker++;             				// Anzeige von Messwerten
        if (HeizZeit> 0) HeizZeit--;        // Verbleibende Heizzeit verringern
        if (LatenzZeit > 0) LatenzZeit--;		// Latenzzeit verringern

        // Anzeigen zyklisch durchschieben, sofern nicht geblockt
        if ( (!TemperaturanzeigeFix) && (Zyklenticker > Anzeigedauer))
         {
          Temperaturanzeige++;
          if (Temperaturanzeige > 2) Temperaturanzeige = 0;
          Zyklenticker = 0;
         }

       }

   	} // From Timer 1 berlauf

   return;									                            // Zurck zum Hauptprogramm
 }

/************************
 *
 * Systemticker auslesen
 *
 ************************/
unsigned long GetTicker (void)
 {
  return (SystemTicker);
 }

/*************************************
 *
 * Verzgern im Millisekundenbereich
 *
 *************************************/
void Delayms (int time)
 {
  unsigned long ticker;

  ticker = GetTicker ();

  while ( (ticker + (time / 10)) > GetTicker () )
   {
    CLRWDT ();
   }
 }

/*******************************************************
 *
 * Status vom Taster des Druck-Drehgebers zurckliefern
 *
 *******************************************************/
char Drucktaster (void)
 {
  if (RB1) DelayUsISR (500);
  return (RB1);
 }

/*************************
 *
 * Klappert am Enable-Pin
 *
 *************************/
void STROBE (void)
 {
 	E = 1;
 	DelayUsISR (300);
	E = 0;
  DelayUsISR (300);
 	E = 1;
 }

/**************************************************************************************************************
 *
 * Ein Byte zum LCD schreiben im 4-Bit-Modus
 *
 **************************************************************************************************************/
void lcd_write (unsigned char c)
 {
  PORTD &= 0x0F;			        		// Obere 4 Bits auf Null
  PORTD |= (c & 0xF0);  					// Oberes Nibble drauflegen
	DelayUs (50);				        	// Kurz warten

 	STROBE ();                     // Rausschieben
	DelayUs (50);                  // Kurz warten

  PORTD &= 0x0F;			        		// Obere 4 Bits auf Null
  PORTD |= ((c & 0x0F) << 4);  	  // Unteres Nibble drauflegen
  DelayUs (50);                  // Kurz warten
	STROBE();                     // Obere 4 Bits auf Null
 }

/**************************************************************************************************************
 *
 * Cursor ein/ausschalten
 *
 **************************************************************************************************************/
void lcd_cursor_on (char type)
 {
 	RS = 0;		                         // write control bytes
 	if (type) lcd_write (0x0F);	else       // display on, blink curson on
   lcd_write (0x0C);	                   // display on, blink curson off
	Delayms (20);
 }

/**************************************************************************************************************
 *
 * LCD lschen und Cursor auf Null setzen
 *
 **************************************************************************************************************/
void lcd_clear (void)
 {
	RS = 0;
	lcd_write (0x1);
	Delayms (10);
 }


/**************************************************************************************************************
 *
 * String auf LCD ausgeben
 *
 **************************************************************************************************************/
void lcd_puts (const char * s)
 {
	RS = 1;	                                 // write characters
	while(*s) lcd_write(*s++);
 }

/**************************************************************************************************************
 *
 * Ein Zeichen auf dem Display ausgeben
 *
 **************************************************************************************************************/
void lcd_putch (char c)
 {
	RS = 1;	                        // write characters
  PORTD &= 0x0F;			        		// Obere 4 Bits auf Null
  PORTD |= (c & 0xF0);  					// Oberes Nibble drauflegen
	DelayUs (50);			   	        	// Kurz warten
	STROBE();                     // Rausschieben
	DelayUs (50);                   // Kurz warten
  PORTD &= 0x0F;			        		// Obere 4 Bits auf Null
  PORTD |= ((c & 0x0F) << 4);  	  // Unteres Nibble drauflegen
  DelayUs (50);                   // Kurz warten
	STROBE();                     // Obere 4 Bits auf Null
	DelayUs (50);                   // Kurz warten
 }

/**************************************************************************************************************
 *
 * Cursor positionieren
 *
 **************************************************************************************************************/
void lcd_goto (unsigned char pos)
 {
	RS = 0;
	lcd_write (0x80+pos);
 }

/**************************************************************************************************************
 *
 * Kontrast einstellen
 *
 **************************************************************************************************************/
void lcd_kontrast (unsigned char wert)
 {
  char i;

  i = wert & 0b00001111;
	RS = 0;
	lcd_write (0x70+i);
  i = (wert & 0b00110000) >> 4;
	lcd_write (0x5F-3+i);
 }


/**************************************************************************************************************
 *
 * LCD initialisieren
 *
 **************************************************************************************************************/
void lcd_init (void)
 {
  char i = 0;

  E = 1;
  RW = 0;				    // Schreibzugriff
	RS = 0;		        // write control bytes
	Delayms (50);		  // power on delay

  PORTD &= 0x0F;
  PORTD |= 0b00110000;   // Initialisierung 3x 0011 xxxx

  for (i = 0; i < 3; i++)
   {
		Delayms (20);
  	STROBE();
   }

	Delayms (20);

  PORTD &= 0x0F;
  PORTD |= 0b00100000;   // Initialisierung 0010 xxxx (4-Bit Mode)

	STROBE();

	Delayms (10);

  // Grundinitialisierung
  lcd_write (0x29);
  DelayUsISR(50);
  lcd_write (0x14);
  DelayUsISR(50);
  lcd_write (0x78);
  DelayUsISR(50);
  lcd_write (0x5F-3);
  DelayUsISR(50);
  lcd_write (0x6A);
  DelayUsISR(50);
  lcd_write (0x0C);
  DelayUsISR(50);
  lcd_write (0x01);
  DelayUsISR(50);
  lcd_write (0x06);

  lcd_kontrast (38);

 }

/****************************************
 *
 * Speichert die Zhlerstnde im EEPROM
 *
 ****************************************/
void Speicherung (void)
 {

   print (UART1, "Zaehlerstaende speichern.\n\r");
   sprintf (outpbuf, "Handstarts : %d\n\r", Handstarts);
   print (UART1, outpbuf);
   sprintf (outpbuf, "Telestarts : %d\n\r", Telestarts);
   print (UART1, outpbuf);

   // Und rein mit den Bits *Sthn*
   EEPromSchreiben (0,  (char)(Handstarts & 0x00FF));
   EEPromSchreiben (1,  (char)((Handstarts & 0xFF00) >> 8));
   EEPromSchreiben (2,  (char)(Telestarts & 0x00FF));
   EEPromSchreiben (3,  (char)((Telestarts & 0xFF00) >> 8));

   print (UART1, "Gespeichert\n\r");

 }

/***************************
 *
 * Initialisierungsroutinen
 *
 ***************************/
void sysinit (void)
 {
  char Tastergriffel;
  int i;

  //--------------------
  // IO-Ports klarmachen
  //--------------------

  // Alle Ports initialisieren bzw. auf Null setzen
  PORTA = 0;
  PORTB = 0;
  PORTC = 0;
  PORTD = 0;
  PORTE = 0;
  PORTF = 0;

  //-------------------------
  // Comperatoren abschalten
  //-------------------------
  CMCON = 0x07;

  // Ein- Ausgangsdefinitionen, 0 = Output, 1 Input, siehe Schaltplan. Nicht benutztes auf Ausgang
  TRISA = 0b00000000;                      //  A0 = Batterienomitor
  TRISB = 0b00001111;                      //  B0= Handy, B2-B3=Drehgeber, B1 = Drucktaster, B4 = Licht einschalten
  TRISC = 0b00000100;                      //  C0 = Heizung, C1=Versorgung, C2 = Zndung
  TRISD = 0b00000000;                      //  D0=E, D1=RW, D2=RS, D4-D7=Datenbus LCD
  TRISE = 0b00000000;                      //
  TRISF = 0b00000000;                      //
  TRISG = 0b00000100;                      //

  // Status Drucktaster festhalten, damit sichere Erkennung mglich
  Tastergriffel = Taster;

  // Versorgung dauerhaft einschalten und erstmal Licht machen
  SaftAn = TRUE;													 // Versorgung stabilisieren
  LichtAn = TRUE;													 // Knips die Funzel an..

  //-----------------
  // UARTS klarmachen
  //-----------------
  UARTInit ();
  CLRWDT();

  //----------------
  // I2C klarmachen
  //----------------
  i2c_init ();

  //-------------------------------------
  // Timer 1 als Systemticker einrichten
  //-------------------------------------
  TMR1IE=1;									                        // enable Interrupt of TMR1
  T1CON=49;									                        // Prescaler=1:8 + Timer on (49)

  //----------------------------
  // Int2 fr Druck-Drehgeber
  //----------------------------
  INTEDG2 = 1;																			// Int2 auf steigender Flanke
  INT2IE = 0;																				// Int2 einschalten
  INT2IF = 1;																				// Int2 einschalten

  //--------------------
  // Alle Interrupts an
  //--------------------
  PEIE=1;          							                    // Peripherie Int=on
  GIE=1;										                        // Define Interupts

  //---------------------
  // Sach ma Beschaid ey
  //---------------------
  print (UART1, "Bordmonitor V1.0\n\r");
  print (UART1, "(C) Fingers elektrische Welt 2010\n\r");
  CLRWDT();

  memset (UARTString, sizeof (UARTString), 0);

  // Gespeicherter Werte reinholen
  Handstarts = EEPromLesen(0);
  Handstarts += ((int)EEPromLesen(1))*256;
  Telestarts = EEPromLesen(2);
  Telestarts += ((int)EEPromLesen(3))*256;
  if ( (Handstarts > 9999) || (Handstarts < 0) ) Handstarts = 0;
  if ( (Telestarts > 9999) || (Telestarts < 0) ) Telestarts = 0;

   print (UART1, "Zaehlerstaende geladen\n\r");
   sprintf (outpbuf, "Handstarts : %d\n\r", Handstarts);
   print (UART1, outpbuf);
   sprintf (outpbuf, "Telestarts : %d\n\r", Telestarts);
   print (UART1, outpbuf);

  //---------------
  // Zeigi fttern
  //---------------
  lcd_init ();
  // lcd_puts ("Hallo Christian");
  print (UART1, "LCD initialisiert\n\r");

  //-----------------------------------
  // Warum ist das Ding berhaupt an ?
  // Wer wagt es die Ruhe zu stren ?
  //-----------------------------------
  if (Zuendung) print (UART1, "Zuendung ein\n\r"); else
   if (Anruf) print (UART1, "Handlich bimmelt\n\r"); else
    if (Tastergriffel) print (UART1, "Griffel am Druecker\n\r");

 if (i2c_BusScan (0x90)) print (UART1, "Innensensor gefunden\n\r"); else print (UART1, "Innensensor nicht gefunden\n\r");


//  BatMonEin = TRUE;        // Batteriemonitor ausknipsen
//

// while (1)
// {
          /*
  for (i=0; i<40; i++)
   {
    sprintf (outpbuf, "%d ", UARTString [i]);
    print (UART1, outpbuf);
   }    */
    // print (UART1, UARTString);


 //  sprintf (outpbuf, "%d \n\r", Pufferzeiger);
 //   print (UART1, outpbuf);
         /*
UARTString [1] = ' ';
UARTString [4] = ' ';
UARTString [13] = ' ';
UARTString [23] = ' ';
       */
//print (UART1, UARTString);

//if (Zerlegung) print (UART1, " Ja"); else print (UART1, " Nein");
//Zerlegung = FALSE;

//   Delayms (500);
//   CLRWDT();
//  }
 }

/*************************************
 *
 * Messwerte auf dem Display anzeigen
 *
 *************************************/
void ZeigeMesswerteNormal (void)
 {
  int Temp = 0;

  if (Zerlegung) BatteriemonitorStringAusbeinen ();

  // Temperaturen reinholen
  Daten.InnenraumTemp = i2c_ReadTemp (0x90);				// Innentemperatur
  Daten.WasserTemp    = i2c_ReadTemp (0x92);				// Externer Wassertemperatursensor

  // Messwerte jede Sekunde aktualisieren
  if (SekundenToggle)
   {
    SekundenToggle = FALSE;

    // Spannung ausgeben
    lcd_goto (0);
    lcd_puts ("  ");
    sprintf (outpbuf, "%3dV  ", (int)(Daten.Spannung * 10));
    outpbuf [4] = outpbuf [3];
    outpbuf [3] = outpbuf [2];
    outpbuf [2] = '.';
    lcd_puts (outpbuf);

    // Strom ausgeben
    if (Daten.Strom < 0) sprintf (outpbuf, "-%03dA  ", (int)(Daten.Strom * (-10))); else
                         sprintf (outpbuf, " %03dA  ", (int)(Daten.Strom * 10));
    if (outpbuf [1] == '0')  outpbuf [1] = ' ';
    outpbuf [5] = outpbuf [4];
    outpbuf [4] = outpbuf [3];
    outpbuf [3] = '.';
    lcd_puts (outpbuf);

   // Temperaturen zyklisch anzeigen
   lcd_goto (0x40);
   TemperaturanzeigeFix = TRUE;
   if (Temperaturanzeige == 0) { lcd_puts ("Batterie  "); Temp = Daten.BatterieTemp; }
   if (Temperaturanzeige == 1) { lcd_puts ("Wasser    "); Temp = Daten.WasserTemp; }
   if (Temperaturanzeige == 2) { lcd_puts ("Innenraum "); Temp = Daten.InnenraumTemp; }
   TemperaturanzeigeFix = FALSE;

    if (Temp < 0) sprintf (outpbuf, "-%3d", (Temp * (-1))); else
            			sprintf (outpbuf, " %3d", (Temp));

    lcd_puts (outpbuf);
    outpbuf [0] = 0xDF;
    outpbuf [1] = 'C';
    outpbuf [2] = '\0';
    lcd_puts (outpbuf);
   } // Von if (SekundenToggle)

 }

/*************************************
 *
 * Messwerte auf dem Display anzeigen
 * solange der Mller nachglht
 *
 *************************************/
void ZeigeMesswerteNachkuehlung (void)
 {
  int Temp;

  // Messwerte jede Sekunde aktualisieren
  if (SekundenToggle)
   {
    SekundenToggle = FALSE;

   // Wassertemperatur anzeigen
   memset (outpbuf, sizeof (outpbuf), 0);
   lcd_goto (0x40);
   Temp = Daten.WasserTemp;

    if (Temp < 0) sprintf (outpbuf, "-%3d", (Temp * (-1))); else
            			sprintf (outpbuf, " %3d", (Temp));

    lcd_puts (outpbuf);
    outpbuf [0] = 0xDF;
    outpbuf [1] = 'C';
    outpbuf [2] = '\0';
    lcd_puts (outpbuf);

    // Restlaufzeit anzeigen
    memset (outpbuf, sizeof (outpbuf), 0);
    sprintf (outpbuf, "  %2d:", HeizZeit / 60);		// Minuten
    lcd_puts (outpbuf);
    sprintf (outpbuf, "%02ds", HeizZeit % 60);     // Sekunden
    lcd_puts (outpbuf);

   } // Von if (SekundenToggle)

 }

/*************************************
 *
 * Messwerte auf dem Display anzeigen
 * solange der Mller rchelt
 *
 *************************************/
void ZeigeMesswerteHeizen (void)
 {
  // Spannung Strom
  // Temperatur Laufzeit

  int Temp = 0;

  memset (outpbuf, sizeof (outpbuf), 0);
  if (Zerlegung) BatteriemonitorStringAusbeinen ();

  // Messwerte jede Sekunde aktualisieren
  if (SekundenToggle)
   {
    SekundenToggle = FALSE;

    // Spannung ausgeben
    lcd_goto (0);
    lcd_puts ("  ");
    memset (outpbuf, sizeof (outpbuf), 0);
    sprintf (outpbuf, "%3dV  ", (int)(Daten.Spannung * 10));
    outpbuf [4] = outpbuf [3];
    outpbuf [3] = outpbuf [2];
    outpbuf [2] = '.';
    lcd_puts (outpbuf);

    // Strom ausgeben
    memset (outpbuf, sizeof (outpbuf), 0);
    if (Daten.Strom < 0) sprintf (outpbuf, "-%03dA  ", (int)(Daten.Strom * (-10))); else
                         sprintf (outpbuf, " %03dA  ", (int)(Daten.Strom * 10));
    if (outpbuf [1] == '0')  outpbuf [1] = ' ';
    outpbuf [5] = outpbuf [4];
    outpbuf [4] = outpbuf [3];
    outpbuf [3] = '.';
    lcd_puts (outpbuf);

   // Wassertemperatur anzeigen

   memset (outpbuf, sizeof (outpbuf), 0);
   lcd_goto (0x40);
   Temp = Daten.WasserTemp;

    if (Temp < 0) sprintf (outpbuf, "-%3d", (Temp * (-1))); else
            			sprintf (outpbuf, " %3d", (Temp));

    lcd_puts (outpbuf);
    outpbuf [0] = 0xDF;
    outpbuf [1] = 'C';
    outpbuf [2] = '\0';
    lcd_puts (outpbuf);


    // Restlaufzeit anzeigen
    memset (outpbuf, sizeof (outpbuf), 0);
    sprintf (outpbuf, "  %2d:", HeizZeit / 60);		// Minuten
    lcd_puts (outpbuf);
    sprintf (outpbuf, "%02ds", HeizZeit % 60);     // Sekunden
    lcd_puts (outpbuf);


   } // Von if (SekundenToggle)

 }


/**********************************
 *
 * Knips der Zecke mal den Saft ab
 *
 **********************************/
void DerLetztemachtdasLichtaus (void)
 {
  print (UART1, "Ich geh jetzt knacken!\n\r");
  BatMonEin = FALSE;        // Batteriemonitor ausknipsen
  LichtAn = FALSE;					// Licht aus oder ich beiss die Lampe ab.....
  HeizungEin = FALSE;		    // Jetzt ist Sommer!
  SaftAn = FALSE;						// Jetzt blst er sich selbst das Licht aus......
  CLRWDT();
 }

/***********************************************************
 *
 * Legt die Bchse schlafen, sofern die Bedingungen stimmen
 *
 ***********************************************************/
char Schlafen (void)
 {
  print (UART1, "Zustand : Schlafen\n\r");

  // Oder doch nicht ????
  if (Zuendung) return (ZuendungEin);
  if (Taster)   return (TastenDruck);
  if (Anruf)
   {
    Telestarts++;
    Speicherung ();
    return (Telestart);
   }

  print (UART1, "Nix los hier.\n\r");

  // Keiner der Zustnde ist eingetreten. Also legen wir uns brav schlafen......
  DerLetztemachtdasLichtaus ();

  while (1);		// Jetzt sind 1.6 Sekunden Zeit, sonst kommt der Wachkter.....
 }

/*******************************
 *
 * Motor luft, Daten anzeigen
 *
 *******************************/
char Normalbetrieb (void)
 {
  char Abbruch = FALSE;
  char ZuendungAusEnde = FALSE;

  print (UART1, "Zustand : Normalbetrieb\n\r");

  lcd_clear ();
  lcd_puts ("Zuendung ein");
  Delayms (1000);

  //---------------------------------------------------------------------------------
  // Nur Zndung abschalten fhrt hier wieder raus. Heizung aus, Batteriemonitor ein
  //---------------------------------------------------------------------------------
  while (!Abbruch)
   {
     CLRWDT();
     ZeigeMesswerteNormal ();

     /*
     if (DrehPosition != AltDrehPosition)
      {
       sprintf (outpbuf, "%d %d\n\r", DrehPosition, AltDrehPosition);
       print (UART1, outpbuf);
       AltDrehPosition = DrehPosition;
      }
      */

    if (Zuendung)
     {
      ZuendungAusEnde = FALSE;
      LatenzZeit = 0;
     }

    if (!Zuendung)
     {
      if (!ZuendungAusEnde) LatenzZeit = DefaultZuendungausZeit;
      ZuendungAusEnde = TRUE;
     }

     // Kam zu lange nichts verwertbares vom Batteriemonitor ? Dann mal zurcksetzen
     if (BatMonTimeout == 0)
      {
       CLRWDT ();
       BatMonEin = FALSE;
       Delayms (500);
       CLRWDT ();
       BatMonTimeout = BatMonTimeoutDefault;
       BatMonEin = TRUE;
      }

    // Abbruchbedingungen
    if ( (!Zuendung) && (LatenzZeit ==0) ) Abbruch = TRUE;
    if ( (!Zuendung) && (Drucktaster ()) ) Abbruch = TRUE;

   } // Von while (Zuendung)

  print (UART1, "Normabetrieb beendet\n\r");
  return (ZuendungAus);
 }

/*************************
 *
 * Zhlerstnde auskotzen
 *
 *************************/
void ZeigeZaehler (void)
 {
   print (UART1, "Zaehlerstaende\n\r");
   sprintf (outpbuf, "Handstarts : %d\n\r", Handstarts);
   print (UART1, outpbuf);
   sprintf (outpbuf, "Telestarts : %d\n\r", Telestarts);
   print (UART1, outpbuf);

   lcd_clear ();
   sprintf (outpbuf, "Handstarts: %d", Handstarts);
   lcd_puts (outpbuf);
   lcd_goto (0x40);
   sprintf (outpbuf, "Telestarts: %d", Telestarts);
   lcd_puts (outpbuf);

   Delayms (2000);
 }

/***********************
 *
 * Einstellungen machen
 *
 ***********************/
char Parametrierung (void)
 {
  char Abbruch = FALSE;
  char HandStart = FALSE;
  int	 Menuepos = 0;

  #define Maxpos  2

  // Heizung aus, Batteriemonitor aus
  print (UART1, "Zustand : Parametrierung\n\r");
  BatMonEin = 0;

  lcd_clear ();
  lcd_puts ("Einstellungen");
  Delayms (1000);

  print (UART1, "Warte auf Loslassung ");
  while (Drucktaster ())
   {
    CLRWDT();
    print (UART1, ".");
    Delayms (250);
   }
  print (UART1, ".. Na endlich!\n\r");

  // Man kann man was einstellen ? Ist das wie IIIbi oder was ?
  HeizZeit = DefaultEinstellZeit;
  INT2IE = 1;																				// Int2 einschalten

  lcd_clear ();
  lcd_puts ("Z");outpbuf [0] = 0x84;outpbuf [1] = '\0';lcd_puts (outpbuf);lcd_puts ("hler ausgeben");
  lcd_goto (0x40);
  lcd_puts ("<Klick>");
  while (Abbruch == FALSE)
   {
     CLRWDT();

     if (DrehPosition != AltDrehPosition)
      {
       HeizZeit = DefaultEinstellZeit;							// Timeout zurcksetzen

       sprintf (outpbuf, "Neu : %d Alt: %d ",DrehPosition, AltDrehPosition);
       print (UART1, outpbuf);

       // Neuen Menpunkt aufbauen
       if (DrehPosition > AltDrehPosition) Menuepos++;
       if (DrehPosition < AltDrehPosition) Menuepos--;

       if (Menuepos >= Maxpos) Menuepos = 0;
       if (Menuepos < 0) 			Menuepos = Maxpos-1;

       sprintf (outpbuf, "Pos : %d\n\r",Menuepos);
       print (UART1, outpbuf);

       // lcd_clear ();
       lcd_goto (0);
       if (Menuepos == 0) { lcd_puts ("Z");outpbuf [0] = 0x84;outpbuf [1] = '\0';lcd_puts (outpbuf);lcd_puts ("hler ausgeben"); };
       if (Menuepos == 1) { lcd_puts ("Manuell starten"); };

       Delayms (200);
       AltDrehPosition = DrehPosition;

      }

     // Hatta Klicke macht der Kleene ?
     if (Drucktaster ())
      {
       print (UART1, "Klicks......\n\r");
       HeizZeit = DefaultEinstellZeit;							// Timeout zurcksetzen
       if (Menuepos == 0)
        {
         ZeigeZaehler ();														// Zhlerstnde auskotzen
         lcd_clear ();
         if (Menuepos == 0) { lcd_puts ("Z");outpbuf [0] = 0x84;outpbuf [1] = '\0';lcd_puts (outpbuf);lcd_puts ("hler ausgeben"); };
         lcd_goto (0x40);
         lcd_puts ("<Klick>");
        }
       if (Menuepos == 1) HandStart = TRUE;					// Heizung starten

      }

     // Wie lufts denn so ?
     if (HeizZeit == 0) Abbruch = ZeitAbgelaufen;								// Zeit ist abgelaufen
     if (Zuendung) Abbruch = ZuendungEin;												// Der Arsch will wohl losgurken
     if (HandStart)
      {
       Abbruch = ManuellerStart;										// Los reiss an dat Ding. Mir frieren gleich die Eier ab!!!!
       Handstarts++;
       Speicherung ();
      }
   }

  INT2IE = 0;																				// Int2 einschalten

  // return (ZeitAbgelaufen);
  return (Abbruch);

 }

/*****************************
 *
 * Heizung aus, Nachkhlphase
 *
 *****************************/
char Nachkuehlen (void)
 {
  char Abbruch = FALSE;

  print (UART1, "Zustand : Nachkuehlen\n\r");

  // Heizung aus, Batteriemonitor aus
  BatMonEin = 0;
  HeizungEin = 0;

  lcd_clear ();

  lcd_puts ("Nachk");
  outpbuf [0] = 0x81;
  outpbuf [1] = '\0';
  lcd_puts (outpbuf);
  lcd_puts ("hlen");

  HeizZeit = DefaultKuehlZeit;

  // Warten bis losgelassen
  while (Drucktaster ());
  Delayms (200);

  while (Abbruch == FALSE)
   {
     CLRWDT();
     ZeigeMesswerteNachkuehlung ();

     // Wie lufts denn so ?
     if (HeizZeit == 0) Abbruch = ZeitAbgelaufen;								// Zeit ist abgelaufen
     if (Zuendung) Abbruch = ZuendungEin;												// Der Arsch will wohl losgurken
     if (Drucktaster ()) Abbruch = ManuellesEnde;								// Verzweigen in Einstellungen
   }

  // Heizung abschalten
  if (Abbruch) print (UART1, "Nachkuehlung beendet\n\r");

  return (Abbruch);
 }

/*****************************************
 *
 * Der Heizmller bollert wie blde.....
 *
 *****************************************/
char HeizungLaeuft (void)
 {
  char Abbruch = FALSE;

  // Wie komme ich hier blo wieder weg ? *Grbel*
  // Knopfdruck, Zndung ein, Zeit abgelaufen, Telestop. Das muss jetzt reichen Kollege!

  // Heizung ein, Batteriemonitor aus
  print (UART1, "Zustand : Heizen\n\r");

  lcd_clear ();
  lcd_puts ("Heizung bollert");
  // Heizung und Monitor einschalten
  BatMonEin = 1;
  HeizungEin = 1;
  Delayms (1000);

  HeizZeit = DefaultHeizZeit;
  LatenzZeit = DefaultLatenzZeit;

  while (Abbruch == FALSE)
   {
     CLRWDT();
     ZeigeMesswerteHeizen ();

     // Wie lufts denn so ?
     if (HeizZeit == 0) Abbruch = ZeitAbgelaufen;								// Zeit ist abgelaufen
     if (Taster) Abbruch = ManuellesEnde;											  // Manuell abgebrochen
  	 if ((Anruf) && (LatenzZeit == 0)) Abbruch = TeleEnde;			// Telestart abgebrochen
     if (Zuendung) Abbruch = ZuendungEin;												// Der Arsch will wohl losgurken
   }

  // Heizung abschalten
  HeizungEin = 0;
  HeizZeit = 0;
  if (Abbruch) print (UART1, "Heizen beendet\n\r");

  return (Abbruch);
 }


/*************
 *
 * Main
 *
 *************/
void main (void)
{
  CLRWDT();

  //----------------------
  // System initialisieren
  //----------------------
  sysinit ();


  //---------------
  // Hauptschleife
  //---------------
  while (1)
   {
     BatMonEin = TRUE;
     CLRWDT();

     // Sind Daten vom Batteriemonitor verfgbar ?
     if (Zerlegung) BatteriemonitorStringAusbeinen ();

     //-----------------
     // Zustandsautomat
     //-----------------
     switch (Zustand)
      {
       case ZustandSchlafen 	: {
       														Rueckgabe = Schlafen ();
       														sprintf (outpbuf, "Schlafen liefert %d zurueck\n\r", Rueckgabe);
       														print (UART1, outpbuf);

       														switch (Rueckgabe)
       														 {
       														  case ZuendungEin : { Zustand = ZustandBetrieb; break; }
																		case TastenDruck : { Zustand = ZustandParameter; break; }
																		case Telestart   : { Zustand = ZustandHeizen; break; }
       														 } // Von switch (Rueckgabe)
       														break;
       													}

       case ZustandBetrieb 		: {
       														Rueckgabe = Normalbetrieb ();
       														sprintf (outpbuf, "Normalbetrieb liefert %d zurueck\n\r", Rueckgabe);
       														print (UART1, outpbuf);

       														switch (Rueckgabe)
       														 {
       														  case ZuendungAus : { Zustand = ZustandSchlafen; break; }
       														 } // Von switch (Rueckgabe)
       														break;
       													}

       case ZustandParameter 	: {
       														Rueckgabe = Parametrierung ();
      														sprintf (outpbuf, "Parametrierung liefert %d zurueck\n\r", Rueckgabe);
                                  print (UART1, outpbuf);

       														switch (Rueckgabe)
       														 {
       														  case ZuendungEin 		: { Zustand = ZustandBetrieb; break; }
       														  case ZeitAbgelaufen : { Zustand = ZustandSchlafen; break; }
																		case ManuellerStart : { Zustand = ZustandHeizen; break; }
       														 } // Von switch (Rueckgabe)

       														break;
       													}

       case ZustandHeizen 	: {
       														Rueckgabe = HeizungLaeuft ();
      														sprintf (outpbuf, "Heizen liefert %d zurueck\n\r", Rueckgabe);

       														switch (Rueckgabe)
       														 {
          													 case ZuendungEin 		: { Zustand = ZustandBetrieb; break; }
          													 case ZeitAbgelaufen : { Zustand = ZustandKuehlen; break; }
          													 case ManuellesEnde  : { Zustand = ZustandParameter; break; }
          													 case TeleEnde       : { Zustand = ZustandKuehlen; break; }
          													 break;
          													}
                              }
       case ZustandKuehlen 	: {
       														Rueckgabe = Nachkuehlen ();
      														sprintf (outpbuf, "Nachkuehlen liefert %d zurueck\n\r", Rueckgabe);

       														switch (Rueckgabe)
       														 {
       														  case ZuendungEin 		: { Zustand = ZustandBetrieb; break; }
       														  case ZeitAbgelaufen : { Zustand = ZustandSchlafen; break; }
       														  case ManuellesEnde : { Zustand = ZustandParameter; break; }
       														  break;
       														 }
       													}


      } // Von switch (Zustand)


   } // Von while (1)


  // Die Bchse endgltig abschalten, gelle ?
  DerLetztemachtdasLichtaus ();
  print (UART1, "ICH WILL NICHT SCHLAFEN\n\r");
  while (1);

}






