/**************************************************************************************************************
 *
 * PIC18F6620-Modul
 *
 * Datei : serial.c
 *
 * Autor : Finger
 *
 * Controller : Microchip 18F6620, 40 Mhz
 * Einstellungen:  OSC=HS, WDT=OFF, PWT=ON, CP=OFF, BOD=ON, LVP=OFF
 *
 *************************************************************************************************************/

#include "serial.h"
#include "defines.h"
#include <pic18.h>
#include <string.h>

#define Empfangspuffergroesse 15

char usart_empfangspuffer [2] [Empfangspuffergroesse];      // Puffer fr empfangene Zeichen
char usart_schreibzeiger [2] = { 0 , 0 };                   // Zeiger auf den Empfangspuffer (schreiben)
char usart_lesezeiger [2] = { 0 , 0 } ;                     // Zeiger auf den Empfangspuffer (lesen)

char cdummy;                                                // Empfangsregister auslesen
char zeiger = 0;                                            // Lasst uns in die Zukunft schauen....

/*************************************************
 *
 * USART-ISR, hngt im Main in der zentralen ISR
 *
 *************************************************/
void isr_usart (void)
 {
   //----------------------------
   // Receive complete-Interrupt
   //----------------------------

   //-------
   // UART 0
   //-------
   if (RCIF1)
    {
     // Nchste Schreibposition ermitteln
     zeiger = usart_schreibzeiger [0];
     zeiger++;
     if (zeiger == Empfangspuffergroesse) zeiger = 0;

     // Ist das Pufferende erreicht ? Dann Zeichen verwerfen
     if (zeiger == usart_lesezeiger [0]) cdummy = RCREG1; else
      {
       usart_empfangspuffer [0] [usart_schreibzeiger [0] ] = RCREG1;    // Zeichen speichern
       usart_schreibzeiger [0] = zeiger;                                // Neuen Zeiger setzen
      }

    } // From if (RCIF)

   //-------
   // UART 1
   //-------
/*
   if (RC2IF)
    {
     // Nchste Schreibposition ermitteln
     zeiger = usart_schreibzeiger [1];
     zeiger++;
     if (zeiger == Empfangspuffergroesse) zeiger = 0;

     // Ist das Pufferende erreicht ? Dann Zeichen verwerfen
     if (zeiger == usart_lesezeiger [1]) cdummy = RCREG2; else
      {
       usart_empfangspuffer [1] [usart_schreibzeiger [1] ] = RCREG2;    // Zeichen speichern
       usart_schreibzeiger [1] = zeiger;                                // Neuen Zeiger setzen
      }

    } // From if (RCIF)
    */
 }

/*****************************************************************************************************************
 *
 * String nach USART schreiben, nicht ISR-gesteuert
 *
 *****************************************************************************************************************/
void print (char UARTNo, const char *data)
{
  char cdummy;                                   // Dummyaufnahme des Empfangsregister

  if (UARTNo == 0)
   { // Ersten UART ansprechen
    if (FERR1) cdummy = RCREG1;
    if (OERR1)
    	{
      	CREN1=0;
      	CREN1=1;
    	}
    	do
    	 {
    		while (!TRMT1);
      	TXREG1 = *data;
      	data++;
    	 } while ( *data /* *data++ */);
   } else // Von if (UARTNo == 0)

   { // Zweiten UART ansprechen
    if (FERR2) cdummy = RCREG2;
    if (OERR2)
    	{
      	CREN2=0;
      	CREN2=1;
    	}
    	do
    	 {
    		while (!TRMT2);
      	TXREG2 = *data;
      	data++;
    	 } while ( *data /* *data++ */);

   } // Von else if (UARTNo == 0)

}

/*****************************************************************************************************************
 *
 * Char nach USART schreiben, nicht ISR-gesteuert
 *
 *****************************************************************************************************************/
void printchar (char UARTNo, char data)
{
  char cdummy;                                   // Dummyaufnahme des Empfangsregister

  if (UARTNo == 0)
   { // Ersten UART ansprechen
    if (FERR1) cdummy = RCREG1;
    if (OERR1)
    	{
      	CREN1=0;
      	CREN1=1;
    	}
     while (!TRMT1);
     TXREG1 = data;
   } else // Von if (UARTNo == 0)

   {
    // Zweiten UART ansprechen
    if (FERR2) cdummy = RCREG;
    if (OERR2)
    	{
      	CREN2=0;
      	CREN2=1;
    	}
     while (!TRMT2);
     TXREG2 = data;
   }  // Von else if (UARTNo == 0)

}

/*****************************************************************************************************************
 *
 * Zeichen aus USART abholen, liefert 0, falls kein Zeichen vorhanden
 *
 *****************************************************************************************************************/
char get_char (char UARTNo)
 {
  char dummy = 0;

  // Sind berhaupt Zeichen vorhanden ?
  if (usart_lesezeiger [UARTNo] != usart_schreibzeiger [UARTNo])
   {
    dummy = usart_empfangspuffer [UARTNo] [usart_lesezeiger [UARTNo] ];
    usart_lesezeiger [UARTNo]++;
    if (usart_lesezeiger [UARTNo] == Empfangspuffergroesse) usart_lesezeiger [UARTNo] = 0;
    return (dummy);
   } else return (0);

 }

/*****************************************************************************************************************
 *
 * Puffer leeren
 *
 *****************************************************************************************************************/
void flushusart (char UARTNo)
 {
  usart_lesezeiger [UARTNo] = usart_schreibzeiger [UARTNo];
 }


/*****************************************************************************************************************
 *
 * UART initialisieren
 *
 *****************************************************************************************************************/
void UARTInit (void)
 {
  // UART-Einstellungen

  // Baudraten
  SPBRG1=33;       							                  // USART1: 19200 Baud bei 10MHZ - 8Bit-Mode mit BRGH=1
  SPBRG2=65;       							                  // USART2: 9600 Baud bei 10MHZ - 8Bit-Mode mit BRGH=1

  // Statusregister
  TXSTA1=0x24;									                    // UART1 : BRGH=1 / TXEN=1
  TXSTA2=0x24;									                    // UART2 : BRGH=1 / TXEN=1

  RCSTA1=0x90;                                      // UART1 : SPEN=1 / CREN=1
  RCSTA2=0x90;                                      // UART2 : SPEN=1 / CREN=1

  BRGH = 1;

  // Interrupts freigeben
  RCIE1=1;										                      // UART1 : Receive-Interrupt ein
  RC2IE=1;										                      // UART2 : Receive-Interrupt ein

  RCIF1=0;										                      // Flag=0;
  RC2IF=0;										                      // Flag=0;
 }


